;------------------------------------------------------------------
; A simple GUI Window that draws a message
;------------------------------------------------------------------

from kernel32.dll
	import	ExitProcess
	import	GetModuleHandleA
	

from user32.dll
	import	MessageBoxA
	import	DefWindowProcA
	import	CreateWindowExA
	import	ShowWindow
	import	UpdateWindow
	;import	DestroyWindow
	import	RegisterClassA

	import	GetMessageA
	import	TranslateMessage
	import	DispatchMessageA
	import	PostQuitMessage
	import	LoadCursorA
	import	LoadIconA
	import	BeginPaint
	import	EndPaint

from gdi32.dll	
	import	TextOutA


section "code" 		class_code
section "data"  	class_data
section "idata" 	class_imports



include ..\win32_inc.asm


.data

	wnd_title	db		"SOL_ASM Test Application",0
	class_name	db		"SOL_Window_Class",0

	msg_text	db		"GUI Hello World!",0
	msg_len		equ		$-msg_text-1			;length without zero

	align 4

	module_handle	dd		0
	main_handle	dd		0

	my_class	WNDCLASS	?

	my_msg		MSG		?

	my_ps		PAINTSTRUCT	?

	msg_end		db		"end",0



.entry App_Init

.code


App_Init:

	invoke	GetModuleHandleA,0
	mov	[module_handle],eax

	invoke	Win_Main

	invoke	ExitProcess,0
	ret


;---------------------------
; Window callback procedure
;---------------------------
PROC Wnd_Proc1
	USES	ebx,esi,edi
	ARG	wnd_handle, wnd_msg, wnd_wparam, wnd_lparam
	LOCAL	tmp_dc


	mov	eax,[wnd_msg]

	.IF eax == WM_DESTROY

		invoke	PostQuitMessage,0
		xor	eax,eax

	.ELSEIF eax == WM_PAINT

		invoke	BeginPaint,[wnd_handle], my_ps
		mov	[tmp_dc],eax

		invoke	TextOutA,eax,32,32,msg_text,msg_len

		invoke	EndPaint,[wnd_handle], my_ps

		xor	eax,eax

	.ELSE
		;----------------------
		; default handling
		;----------------------
		invoke	DefWindowProcA,[wnd_handle],[wnd_msg],[wnd_wparam],[wnd_lparam]		
	.ENDIF


	ret

ENDP


;------------------------------------
; This creates our window
; and then it enters message loop
;------------------------------------

PROC Win_Main stdcall
	USES	esi

	;------------------------------------
	; fill-up and register the class
	;------------------------------------
	mov	esi,  my_class
	
	mov	eax,CS_HREDRAW
	mov	[esi + WNDCLASS.style],CS_HREDRAW

	mov	eax, Wnd_Proc1
	mov	[esi + WNDCLASS.wnd_proc_ptr], eax

	mov	eax,[module_handle]
	mov	[esi + WNDCLASS.instance_handle],eax

	mov	eax, class_name
	mov	[esi + WNDCLASS.class_name_ptr],eax

	;mov	eax,COLOR_WINDOW
	;mov	eax,COLOR_BTNSHADOW
	mov	eax,COLOR_BTNFACE
	add	eax,2
	
	mov	[esi + WNDCLASS.back_brush_handle],eax

	invoke	LoadCursorA,0,IDC_ARROW
	;invoke	LoadCursorA,0,IDC_CROSS
	mov	[esi + WNDCLASS.cursor_handle],eax

	invoke	LoadIconA,0,IDI_APPLICATION
	mov	[esi + WNDCLASS.icon_handle],eax

	invoke	RegisterClassA,esi


	;-----------------------------
	; create the window
	;-----------------------------
	mov	eax,WS_OVERLAPPED+WS_CAPTION+WS_SYSMENU+WS_THICKFRAME+WS_MINIMIZEBOX+WS_MAXIMIZEBOX

	invoke	CreateWindowExA,0,class_name,wnd_title,eax,32,64,320,240,0,0,[module_handle],0
	mov	[main_handle],eax

	invoke	ShowWindow, eax, SW_SHOW

	invoke	UpdateWindow,[main_handle]


	;----------------------------
	; windows message loop
	;----------------------------

win_msg_loop:

	invoke	GetMessageA,my_msg,0,0,0

	test	eax,eax
	jz	finish

	cmp	eax,0ffffffffh
	jz	finish

	invoke	TranslateMessage,my_msg
	invoke	DispatchMessageA,my_msg			

	jmp	win_msg_loop


finish:

	ret
ENDP














